comment  
[SALC - Set AL on Carry]                               [Assembler][/][8086]

This small gem presents an undocumented opcode available on all Intel CPUs.
The instruction mnemonic is called SALC and sets AL to match the carry
flag, ie if the Carry is set, AL will be set to FF, else zero. The
 instruction macro looks like this:

 salc - set al to carry (undocumented opcode)

 input:
   none
 output:
   al = 0FFh * CF
 destroys:
   nothing

It does not affect any flags. It was first documented officially when Intel
released the Pentium Pro processor. It is officially named SALC.
                                                  Gem writer: John Eckerdal
                                                   last updated: 1998-03-16
found in http://www.azillionmonkeys.com/qed/asm.html   by Paul Hsieh       
This instruction can have others mnemonics like SETALC or SETCAL


; 
    include \masm32\include\masm32rt.inc
    .686
    include \masm32\macros\timers.asm
; 
Assume fs:nothing

salc macro
	db 0d6h 
endm

    .code
; 
start:
	push offset exception_handler		;install SEH to catch invalid opcode
	push fs:[0]
	mov fs:[0],esp

;-----------------here start our program-------------------------------------
					;xor eax,eax		;zero eax
					;stc				;set carry
					;sbb eax,eax		;eax = 0ffffffffh
					;xor eax,eax		;eax = 0
					;stc				;set carry
					;sbb al,al		;eax = 000000ffh
	push 1
	call ShowCpu			; print brand string and SSE level
	
	xor eax,eax				;zero flag = 1
	cmp eax,1				;move zero flag to carry
	salc					;if carry, al = 0ffh
	or eax,eax				;cmp eax,0
	jz do_not_support		;not suported
	sub eax,eax				;eax = 0, CF = 0
	mov al,0ffh				;eax = 000000ffh
	salc					;al = 00h
	cmp eax,0				;eax = 00?
	jne	do_not_support		;not suported
	jmp get_cycles			;supported, get cycles
	do_not_support:
	print "do not support undocumented SALC instruction",13,10
	jmp quit
	get_cycles:
; 
    invoke Sleep, 3000

    counter_begin 1000, HIGH_PRIORITY_CLASS
      REPEAT 100
      stc
      salc
      ENDM
    counter_end
    print ustr$(eax)," cycles, salc REPEAT 100",13,10

quit:
    counter_begin 1000, HIGH_PRIORITY_CLASS
      REPEAT 100
      stc
      SBB AL,AL
      ENDM
    counter_end
    print ustr$(eax)," cycles, sbb al,al REPEAT 100",13,10

;    counter_begin 1000, HIGH_PRIORITY_CLASS
;      REPEAT 100
;      stc
;      SBB eax,eax			;same cycles results that sbb al,al in my pc
;      ENDM
;    counter_end
;    print ustr$(eax)," cycles, sbb eax,eax REPEAT 100",13,10,13,10

    inkey "Press any key to exit..."
    exit
; 

;this procedure was get from jj2007 in strlen algo
ShowCpu proc	; mode:DWORD
COMMENT @ Usage: 
  push 0, call ShowCpu	; simple, no printing, just returns SSE level
  push 1, call ShowCpu	; prints the brand string and returns SSE level@
  pushad
  sub esp, 80	; create a buffer for the brand string
  mov edi, esp		; point edi to it
  xor ebp, ebp
  .Repeat
  	lea eax, [ebp+80000002h]
	db 0Fh, 0A2h	; cpuid 80000002h-80000004h
	stosd
	mov eax, ebx
	stosd
	mov eax, ecx
	stosd
	mov eax, edx
	stosd
	inc ebp
  .Until ebp>=3
  push 1
  pop eax
  db 0Fh, 0A2h		; cpuid 1
  xor ebx, ebx		; CpuSSE
  xor esi, esi		; add zero plus the carry flag
  bt edx, 25		; edx bit 25, SSE1
  adc ebx, esi
  bt edx, 26		; edx bit 26, SSE2
  adc ebx, esi
  bt ecx, esi		; ecx bit 0, SSE3
  adc ebx, esi
  bt ecx, 9			; ecx bit 9, SSE4
  adc ebx, esi
  dec dword ptr [esp+4+32+80]	; dec mode in stack
  .if Zero?
	mov edi, esp	; restore pointer to brand string
  	.Repeat				
		.Break .if byte ptr [edi]!=32	; mode was 1, so show a string but skip leading blanks
		inc edi
	.Until 0
	.if byte ptr [edi]<32
		print chr$("pre-P4")
	.else
		print edi	; CpuBrand
	.endif
	.if ebx
		print chr$(32, 40, "SSE")	; info on SSE level, 40=(
		print str$(ebx), 41, 13, 10	; 41=)
	.endif
  .endif
  add esp, 80		; discard brand buffer (after printing!)
  mov [esp+32-4], ebx	; move ebx into eax stack position - returns eax to main for further use
  ifdef MbBufferInit
	call MbBufferInit
  endif
  popad
  ret 4
ShowCpu endp


exception_handler proc public uses esi edi ebx pExcept:dword,pFrame:dword,pContext:dword,pDispatch: dword
assume ebx:ptr CONTEXT
assume eax:ptr EXCEPTION_RECORD
assume edi:ptr ERRO
mov ebx,pContext
mov eax,pExcept
mov edi,pFrame

;.if [eax].ExceptionCode == EXCEPTION_ILLEGAL_INSTRUCTION
	print "do not support undocumented SALC instruction",13,10
	inkey
	exit
;.endif
	ret
exception_handler endp
end start


Intel(R) Pentium(R) Dual  CPU  E2160  @ 1.80GHz (SSE4)
144 cycles, salc REPEAT 100
213 cycles, sbb al,al REPEAT 100



ud2	macro		;to generate test with invalid opcode, SEH handler
db 0fh,0bh		;db 0fh,0b9h
endm